@@ -40,6 +40,7 @@
 
 static void *plugin_handle;
 static bool plugin_loaded;
+static char *execname;
 
 static void
 print_each_ace_mask(uint32_t mask)
@@ -106,10 +107,7 @@ print_ace_flags(uint8_t flags, int raw)
 	}
 
 	if (flags & OBJECT_INHERIT_FLAG) {
-		if (mflags)
-			printf("|");
-		else
-			mflags = true;
+		mflags = true;
 		printf("OI");
 	}
 	if (flags & CONTAINER_INHERIT_FLAG) {
@@ -198,7 +196,7 @@ print_sid_raw:
 	id_auth_val += (unsigned long long)csid->authority[3] << 16;
 	id_auth_val += (unsigned long long)csid->authority[2] << 24;
 	id_auth_val += (unsigned long long)csid->authority[1] << 32;
-	id_auth_val += (unsigned long long)csid->authority[0] << 48;
+	id_auth_val += (unsigned long long)csid->authority[0] << 40;
 
 	/*
 	 * MS-DTYP states that if the authority is >= 2^32, then it should be
@@ -334,6 +332,8 @@ getcifsacl_usage(const char *prog)
 		prog);
 	fprintf(stderr, "Usage: %s [option] <file_name>\n", prog);
 	fprintf(stderr, "Valid options:\n");
+	fprintf(stderr, "\t-h	Display this help text\n");
+	fprintf(stderr, "\n");
 	fprintf(stderr, "\t-v	Version of the program\n");
 	fprintf(stderr, "\n");
 	fprintf(stderr, "\t-r	Display raw values of the ACE fields\n");
@@ -348,8 +348,15 @@ main(const int argc, char *const argv[])
 	ssize_t attrlen;
 	size_t bufsize = BUFSIZE;
 	char *filename, *attrval;
+	execname = basename(argv[0]);
 
-	while ((c = getopt_long(argc, argv, "r:v", NULL, NULL)) != -1) {
+	if (argc < 2) {
+		fprintf(stderr, "%s: you must specify a filename.\n", execname);
+		printf("Try `getcifsacl -h' for more information.\n");
+		goto out;
+	}
+
+	while ((c = getopt_long(argc, argv, "rhv", NULL, NULL)) != -1) {
 		switch (c) {
 		case 'v':
 			printf("Version: %s\n", VERSION);
@@ -358,18 +365,17 @@ main(const int argc, char *const argv[])
 			raw = true;
 			break;
 		default:
-			break;
+			getcifsacl_usage(execname);
+			goto out;
 		}
 	}
 
-	if (raw && argc == 3)
-		filename = argv[2];
-	else if (argc == 2)
-		filename = argv[1];
-	else {
-		getcifsacl_usage(basename(argv[0]));
+	if (optind >= argc) {
+		printf("you must specify a filename after options.\n");
+		printf("Usage: getcifsacl [option] <file_name>\n");
 		goto out;
-	}
+	} else
+		filename = argv[optind];
 
 	if (!raw && !plugin_loaded) {
 		ret = init_plugin(&plugin_handle);
@@ -401,8 +407,12 @@ cifsacl:
 			free(attrval);
 			bufsize += BUFSIZE;
 			goto cifsacl;
-		} else
-			printf("getxattr error: %d\n", errno);
+		} else {
+			fprintf(stderr, "getxattr failed on %s: %s\n", filename, strerror(errno) );
+			free(attrval);
+			ret = -1;
+			goto out;
+		}
 	}
 
 	parse_sec_desc((struct cifs_ntsd *)attrval, attrlen, raw);
