@@ -43,6 +43,7 @@
 #include <limits.h>
 #include <paths.h>
 #include <libgen.h>
+#include <time.h>
 #include <sys/mman.h>
 #include <sys/wait.h>
 #ifdef HAVE_SYS_FSUID_H
@@ -161,10 +162,16 @@
 #define OPT_BKUPUID    30
 #define OPT_BKUPGID    31
 #define OPT_NOFAIL     32
+#define OPT_SNAPSHOT   33
 
 #define MNT_TMP_FILE "/.mtab.cifs.XXXXXX"
 
-/* struct for holding parsed mount info for use by privleged process */
+#define GMT_NAME_LEN 24 /* length of a @GMT- name */
+#define GMT_FORMAT "@GMT-%Y.%m.%d-%H.%M.%S"
+
+#define NTFS_TIME_OFFSET ((unsigned long long)(369*365 + 89) * 24 * 3600 * 10000000)
+
+/* struct for holding parsed mount info for use by privileged process */
 struct parsed_mount_info {
 	unsigned long flags;
 	char host[NI_MAXHOST + 1];
@@ -267,22 +274,32 @@ static int mount_usage(FILE * stream)
 	fprintf(stream,
 		"\n\tsep=<char>,iocharset=<codepage>,suid,nosuid,exec,noexec,serverino,");
 	fprintf(stream,
-		"\n\tmapchars,nomapchars,nolock,servernetbiosname=<SRV_RFC1001NAME>");
+		"\n\tnoserverino,mapchars,nomapchars,nolock,servernetbiosname=<SRV_RFC1001NAME>");
 	fprintf(stream,
-		"\n\tdirectio,nounix,cifsacl,sec=<authentication mechanism>,sign,seal,fsc");
+		"\n\tcache=<strict|none|loose>,nounix,cifsacl,sec=<authentication mechanism>,");
+	fprintf(stream,
+		"\n\tsign,seal,fsc,snapshot=<token|time>,nosharesock,persistenthandles,");
+	fprintf(stream,
+		"\n\tresilienthandles,rdma,vers=<smb_dialect>,cruid");
 	fprintf(stream,
 		"\n\nOptions not needed for servers supporting CIFS Unix extensions");
 	fprintf(stream,
 		"\n\t(e.g. unneeded for mounts to most Samba versions):");
 	fprintf(stream,
-		"\n\tuid=<uid>,gid=<gid>,dir_mode=<mode>,file_mode=<mode>,sfu");
+		"\n\tuid=<uid>,gid=<gid>,dir_mode=<mode>,file_mode=<mode>,sfu,");
+	fprintf(stream,
+		"\n\tmfsymlinks,idsfromsid");
 	fprintf(stream, "\n\nRarely used options:");
 	fprintf(stream,
 		"\n\tport=<tcpport>,rsize=<size>,wsize=<size>,unc=<unc_name>,ip=<ip_address>,");
 	fprintf(stream,
 		"\n\tdev,nodev,nouser_xattr,netbiosname=<OUR_RFC1001NAME>,hard,soft,intr,");
 	fprintf(stream,
-		"\n\tnointr,ignorecase,noposixpaths,noacl,prefixpath=<path>,nobrl");
+		"\n\tnointr,ignorecase,noposixpaths,noacl,prefixpath=<path>,nobrl,");
+	fprintf(stream,
+		"\n\techo_interval=<seconds>,actimeo=<seconds>,max_credits=<credits>,");
+	fprintf(stream,
+		"\n\tbsize=<size>");
 	fprintf(stream,
 		"\n\nOptions are described in more detail in the manual page");
 	fprintf(stream, "\n\tman 8 mount.cifs\n");
@@ -763,6 +780,8 @@ static int parse_opt_token(const char *token)
 		return OPT_NOFAIL;
 	if (strncmp(token, "x-", 2) == 0)
 		return OPT_IGNORE;
+	if (strncmp(token, "snapshot", 8) == 0)
+		return OPT_SNAPSHOT;
 
 	return OPT_ERROR;
 }
@@ -783,16 +802,19 @@ parse_options(const char *data, struct parsed_mount_info *parsed_info)
 	int got_uid = 0;
 	int got_cruid = 0;
 	int got_gid = 0;
+	int got_snapshot = 0;
 	uid_t uid, cruid = 0, bkupuid = 0;
 	gid_t gid, bkupgid = 0;
 	char *ep;
 	struct passwd *pw;
 	struct group *gr;
 	/*
-	 * max 32-bit uint in decimal is 4294967295 which is 10 chars wide
-	 * +1 for NULL, and +1 for good measure
+	 * max 64-bit uint in decimal is 18446744073709551615 which is 20 chars
+	 * wide +1 for NULL, and +1 for good measure
 	 */
-	char txtbuf[12];
+	char txtbuf[22];
+	unsigned long long snapshot;
+	struct tm tm;
 
 	/* make sure we're starting from beginning */
 	out[0] = '\0';
@@ -1120,6 +1142,19 @@ parse_options(const char *data, struct parsed_mount_info *parsed_info)
 		case OPT_NOFAIL:
 			parsed_info->nofail = 1;
 			goto nocopy;
+		case OPT_SNAPSHOT:
+			if (!value || !*value)
+				goto nocopy;
+			if (strncmp(value, "@GMT-", 5))
+				break;
+			if ((strlen(value) != GMT_NAME_LEN) ||
+			    (strptime(value, GMT_FORMAT, &tm) == NULL)) {
+				fprintf(stderr, "bad snapshot token\n");
+				return EX_USAGE;
+			}
+			snapshot = timegm(&tm) * 10000000 + NTFS_TIME_OFFSET;
+			got_snapshot = 1;
+			goto nocopy;
 		}
 
 		/* check size before copying option to buffer */
@@ -1215,7 +1250,7 @@ nocopy:
 	if (got_bkupgid) {
 		word_len = snprintf(txtbuf, sizeof(txtbuf), "%u", bkupgid);
 
-		/* comma + "backkupgid=" + terminating NULL == 12 */
+		/* comma + "backupgid=" + terminating NULL == 12 */
 		if (out_len + word_len + 12 > MAX_OPTIONS_LEN) {
 			fprintf(stderr, "Options string too long\n");
 			return EX_USAGE;
@@ -1227,6 +1262,21 @@ nocopy:
 		}
 		snprintf(out + out_len, word_len + 11, "backupgid=%s", txtbuf);
 	}
+	if (got_snapshot) {
+		word_len = snprintf(txtbuf, sizeof(txtbuf), "%llu", snapshot);
+
+		/* comma + "snapshot=" + terminating NULL == 11 */
+		if (out_len + word_len + 11 > MAX_OPTIONS_LEN) {
+			fprintf(stderr, "Options string too long\n");
+			return EX_USAGE;
+		}
+
+		if (out_len) {
+			strlcat(out, ",", MAX_OPTIONS_LEN);
+			out_len++;
+		}
+		snprintf(out + out_len, word_len + 11, "snapshot=%s", txtbuf);
+	}
 
 	return 0;
 }
@@ -2099,6 +2149,10 @@ mount_retry:
 		switch (errno) {
 		case ECONNREFUSED:
 		case EHOSTUNREACH:
+			if (currentaddress) {
+				fprintf(stderr, "mount error(%d): could not connect to %s",
+					errno, currentaddress);
+			}
 			currentaddress = nextaddress;
 			if (currentaddress) {
 				nextaddress = strchr(currentaddress, ',');
@@ -2110,6 +2164,12 @@ mount_retry:
 			fprintf(stderr,
 				"mount error: %s filesystem not supported by the system\n", cifs_fstype);
 			break;
+		case EHOSTDOWN:
+			fprintf(stderr,
+				"mount error: Server abruptly closed the connection.\n"
+				"This can happen if the server does not support the SMB version you are trying to use.\n"
+				"The default SMB version recently changed from SMB1 to SMB2.1 and above. Try mounting with vers=1.0.\n");
+			break;
 		case ENXIO:
 			if (!already_uppercased &&
 			    uppercase_string(parsed_info->host) &&
@@ -2126,7 +2186,7 @@ mount_retry:
 			strerror(errno));
 		fprintf(stderr,
 			"Refer to the %s(8) manual page (e.g. man "
-			"%s)\n", thisprogram, thisprogram);
+			"%s) and kernel log messages (dmesg)\n", thisprogram, thisprogram);
 		rc = EX_FAIL;
 		goto mount_exit;
 	}
